@@ -1,20 +1,22 @@
 """
-    Copyright (C) 2023  Michael Ablassmeier <abi@grinser.de>
+Copyright (C) 2023  Michael Ablassmeier <abi@grinser.de>
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
-    (at your option) any later version.
+This program is free software: you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <https://www.gnu.org/licenses/>.
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <https://www.gnu.org/licenses/>.
 """
+
 from typing import List, Any, Tuple, IO, Union
+from nbd import Error as nbdError
 from libvirtnbdbackup import block
 from libvirtnbdbackup import lz4
 from libvirtnbdbackup.exceptions import DiskBackupFailed
@@ -48,7 +50,7 @@ def write(
 
         try:
             data = nbdCon.nbd.pread(blocklen, blockOffset)
-        except nbdCon.nbd.Error as e:
+        except nbdError as e:
             raise DiskBackupFailed(e) from e
 
         if compress is not False:
