@@ -19,6 +19,7 @@ import logging
 from typing import List, Any, Generator, Dict
 from nbd import CONTEXT_BASE_ALLOCATION
 from libvirtnbdbackup.objects import Extent, _ExtentObj
+from libvirtnbdbackup.common import humanize
 
 log = logging.getLogger("extenthandler")
 
@@ -32,7 +33,7 @@ class ExtentHandler:
     extent information as nbdinfo or qemu-img map
     """
 
-    def __init__(self, nbdFh, cType) -> None:
+    def __init__(self, nbdFh, cType, no_sparse_detection: bool) -> None:
         self.useQemu = False
         self._maxRequestBlock: int = 4294967295
         self._align: int = 512
@@ -44,6 +45,7 @@ class ExtentHandler:
         self._nbdFh = nbdFh
         self._cType = cType
         self._extentEntries: Dict = {}
+        self.no_sparse_detection = no_sparse_detection
 
         if cType.metaContext == "":
             self._metaContext = CONTEXT_BASE_ALLOCATION
@@ -55,9 +57,12 @@ class ExtentHandler:
             log.debug("NBD server exports [%d] metacontexts:", contexts)
             for i in range(0, contexts):
                 ctx = self._nbdFh.nbd.get_meta_context(i)
+                if self.no_sparse_detection is True and ctx == CONTEXT_BASE_ALLOCATION:
+                    continue
                 self._extentEntries[ctx] = []
         else:
-            self._extentEntries[CONTEXT_BASE_ALLOCATION] = []
+            if self.no_sparse_detection is False:
+                self._extentEntries[CONTEXT_BASE_ALLOCATION] = []
             self._extentEntries[self._metaContext] = []
 
         log.debug("Primary meta context for backup: %s", self._metaContext)
@@ -198,47 +203,67 @@ class ExtentHandler:
             e for e in extents if e.context == self._metaContext and e.data
         ]
 
-        selected_extents = []
         totalLength: int = 0
+        result = []
+        i = 0  # index for base_extents
+        j = 0  # index for backup_extents
+        while i < len(base_extents) and j < len(backup_extents):
+            base = base_extents[i]
+            backup = backup_extents[j]
 
-        for backup in backup_extents:
-            backup_end = backup.offset + backup.length
-            log.debug("check extent backup %d %d", backup.offset, backup_end)
-            for base in base_extents:
-                base_end = base.offset + base.length
-                log.debug(
-                    "add base0 %d %d %d %d",
-                    backup.offset,
-                    base.offset,
-                    base_end,
-                    backup_end,
+            log.debug(
+                "base: %d:%d(%s) bitmap: %d:%d(%s)",
+                base.length,
+                base.offset,
+                str(base.data),
+                backup.length,
+                backup.offset,
+                str(backup.data),
+            )
+
+            # Skip if either extent has data=False or no real intersection
+            if not base.data or base.offset + base.length <= backup.offset:
+                i += 1
+                continue
+            if not backup.data or backup.offset + backup.length <= base.offset:
+                j += 1
+                continue
+
+            offset = max(base.offset, backup.offset)
+            end = min(backup.offset + backup.length, base.offset + base.length)
+            log.debug("-->: %d:%d", offset, end - offset)
+            result.append(
+                Extent(
+                    context=base.context,
+                    data=True,
+                    offset=offset,
+                    length=end - offset,
                 )
-                ext = Extent(base.context, base.data, base.offset, base.length)
-                ext.offset = max(base.offset, backup.offset)
-                ext_end = min(base_end, backup_end)
-                ext.length = max(0, ext_end - ext.offset)
-                if ext.length:
-                    log.debug(
-                        "-> extent %d %d %d",
-                        ext.offset,
-                        ext.offset + ext.length,
-                        ext.length,
-                    )
-                    selected_extents.append(ext)
-                    totalLength += ext.length
+            )
+            totalLength += end - offset
+
+            # advance
+            if end == base.offset + base.length:
+                i += 1
+            if end == backup.offset + backup.length:
+                j += 1
 
         if totalLength > 0:
             log.info(
-                "Detected [%d] sparse bytes for current bitmap.",
+                "Detected %d bytes [%s] non-sparse blocks for current bitmap.",
                 totalLength,
+                humanize(totalLength),
             )
-        return selected_extents
+
+        return result
 
     def queryBlockStatus(self) -> List[Extent]:
         """Check the status for each extent, whether if it is
         real data or zeroes, return a list of extent objects
         """
+        log.info("Start receiving backup extents.")
         extents = self.queryExtents()
+        log.info("Finished receiving extents.")
         extentList: List[Extent] = []
         start: int = 0
         baseStart: int = 0
@@ -264,6 +289,10 @@ class ExtentHandler:
                 extObj.offset,
                 extObj.offset + extObj.length,
             )
+        if self.no_sparse_detection is True:
+            log.info("Skipping detection of sparse/fstrimmed blocks.")
+            return extentList
+
         if self._metaContext != CONTEXT_BASE_ALLOCATION:
             log.debug("Detected [%d] bytes of changed data regions.", totalLength)
             extentList = self.overlap(extentList)
